import { CodegenResult } from '@next-vue/compiler-core';
import { CompilerError } from '@next-vue/compiler-core';
import { CompilerOptions } from '@next-vue/compiler-core';
import { generateCodeFrame } from '@next-vue/compiler-core';
import { LazyResult } from 'postcss';
import { RawSourceMap } from 'source-map';
import { Result } from 'postcss';
import { SourceLocation } from '@next-vue/compiler-core';

declare interface AssetURLOptions {
    [name: string]: string[];
}
export { CompilerOptions }

export declare function compileStyle(options: StyleCompileOptions): StyleCompileResults;

export declare function compileStyleAsync(options: StyleCompileOptions): Promise<StyleCompileResults>;

export declare function compileTemplate(options: TemplateCompileOptions): TemplateCompileResults;
export { generateCodeFrame }

export declare function parse(source: string, { sourceMap, filename, sourceRoot, pad }?: SFCParseOptions): SFCDescriptor;

declare type PreprocessLang = 'less' | 'sass' | 'scss' | 'styl' | 'stylus';

export declare interface SFCBlock {
    type: string;
    content: string;
    attrs: Record<string, string | true>;
    loc: SourceLocation;
    map?: RawSourceMap;
    lang?: string;
    src?: string;
}

export declare interface SFCDescriptor {
    filename: string;
    template: SFCTemplateBlock | null;
    script: SFCScriptBlock | null;
    styles: SFCStyleBlock[];
    customBlocks: SFCBlock[];
}

export declare interface SFCParseOptions {
    filename?: string;
    sourceMap?: boolean;
    sourceRoot?: string;
    pad?: boolean | 'line' | 'space';
}

export declare interface SFCScriptBlock extends SFCBlock {
    type: 'script';
}

export declare interface SFCStyleBlock extends SFCBlock {
    type: 'style';
    scoped?: boolean;
    module?: string | boolean;
}

export declare interface SFCTemplateBlock extends SFCBlock {
    type: 'template';
    functional?: boolean;
}

export declare interface StyleCompileOptions {
    source: string;
    filename: string;
    id: string;
    map?: RawSourceMap;
    scoped?: boolean;
    trim?: boolean;
    preprocessLang?: PreprocessLang;
    preprocessOptions?: any;
    postcssOptions?: any;
    postcssPlugins?: any[];
}

export declare interface StyleCompileResults {
    code: string;
    map: RawSourceMap | undefined;
    rawResult: LazyResult | Result | undefined;
    errors: Error[];
}

export declare interface TemplateCompileOptions {
    source: string;
    filename: string;
    compiler?: TemplateCompiler;
    compilerOptions?: CompilerOptions;
    preprocessLang?: string;
    preprocessOptions?: any;
    transformAssetUrls?: AssetURLOptions | boolean;
}

export declare interface TemplateCompiler {
    compile(template: string, options: CompilerOptions): CodegenResult;
}

export declare interface TemplateCompileResults {
    code: string;
    source: string;
    tips: string[];
    errors: (string | CompilerError)[];
    map?: RawSourceMap;
}

export { }
